#define FIXED_NOISE
#define PROCEDURAL_NOISE

varying mediump vec4 v_color;
#ifdef FIXED_NOISE
    varying highp vec4 v_pos;
#endif
//varying mediump vec2 v_uv;
varying mediump vec2 v_screen_pos;

uniform mediump float u_time;
#ifndef FIXED_NOISE
    uniform mediump vec2 u_offset;
#endif
uniform sampler2D u_noise;
//uniform sampler2D u_color;

//uniform vec2 u_character_pos;

highp float rand(highp vec2 co)
{
#ifdef PROCEDURAL_NOISE
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
#else
    return texture2D(u_noise, co).r;
#endif
}

#if ENABLE_EDITOR
mediump vec3 color_filter(mediump vec3 in_col, mediump float brightness, mediump float contrast, mediump vec3 high_tone, mediump vec3 low_tone, mediump float balance, mediump float saturation, float hue)
{
	in_col = clamp((in_col + vec3(brightness) - vec3(0.5))*contrast + vec3(0.5), vec3(0.0), vec3(1.0));

	mediump float luma = (0.2126*in_col.r + 0.7152*in_col.g + 0.0722*in_col.b)/(0.2126 + 0.7152 + 0.0722);

	mediump float t;
	if (luma > balance)
		t = 0.5 + (luma - balance)/(1.0 - balance)/2.0;
	else
		t = luma/balance/2.0;
	mediump vec3 tone = mix(low_tone, high_tone, t);

	mediump float tone_max = max(max(tone.r, tone.g), tone.b);
	mediump float tone_min = min(min(tone.r, tone.g), tone.b);
	mediump float tone_delta = tone_max - tone_min;
	mediump float tone_lightness = (tone_max + tone_min)/2.0;
	mediump float tone_sat = tone_delta/(1.0 - abs(tone_lightness - 1.0));

	mediump float tone_luma = (0.2126*tone.r + 0.7152*tone.g + 0.0722*tone.b)/(0.2126 + 0.7152 + 0.0722);
	mediump vec3 same_luma_tone = tone/tone_luma*luma;

	mediump vec3 splitted_c = mix(in_col, same_luma_tone, tone_sat);

	// Saturation
	mediump float splitted_luma = (0.2126*splitted_c.r + 0.7152*splitted_c.g + 0.0722*splitted_c.b)/(0.2126 + 0.7152 + 0.0722);
	mediump vec3 saturated = vec3(
		mix(splitted_luma, splitted_c.r, saturation),
		mix(splitted_luma, splitted_c.g, saturation),
		mix(splitted_luma, splitted_c.b, saturation)
	);

	// Hue rotation
    mediump float cosA = cos(hue*3.14159265);
    mediump float sinA = sin(hue*3.14159265);
	mediump float onethirdsqrt = sqrt(1.0/3.0);
    mediump mat3 matrix;
	matrix[0].xyz = vec3(cosA + (1.0 - cosA) / 3.0, 1.0/3.0 * (1.0 - cosA) - onethirdsqrt * sinA, 1.0/3.0 * (1.0 - cosA) + onethirdsqrt * sinA);
    matrix[1].xyz = vec3(1.0/3.0 * (1.0 - cosA) + onethirdsqrt * sinA, cosA + 1.0/3.0*(1.0 - cosA), 1.0/3.0 * (1.0 - cosA) - onethirdsqrt * sinA);
	matrix[2].xyz = vec3(1.0/3.0 * (1.0 - cosA) - onethirdsqrt * sinA, 1.0/3.0 * (1.0 - cosA) + onethirdsqrt * sinA, cosA + 1.0/3.0 * (1.0 - cosA));
	mediump vec3 hued = vec3(
		clamp(saturated[0]*matrix[0][0] + saturated[1]*matrix[0][1] + saturated[2]*matrix[0][2], 0.0, 1.0),
		clamp(saturated[0]*matrix[1][0] + saturated[1]*matrix[1][1] + saturated[2]*matrix[1][2], 0.0, 1.0),
		clamp(saturated[0]*matrix[2][0] + saturated[1]*matrix[2][1] + saturated[2]*matrix[2][2], 0.0, 1.0)
	);

	return hued;
}

uniform mediump float u_brightness;
uniform mediump float u_contrast;
uniform mediump vec3 u_split_high_tone;
uniform mediump vec3 u_split_low_tone;
uniform mediump float u_split_balance;
uniform mediump float u_saturation;
uniform mediump float u_hue;
#endif

void main()
{
#ifndef VERY_LOW_GRAPHICS

	//lowp vec4 tex_col = texture2D(u_color, v_uv);
	lowp vec3 color = v_color.rgb;//*tex_col.rgb;

	//color = pow(color, vec3(1.0/1.8)); // Gamma-conversion baked in editor

#if 1
	mediump float len = dot(v_screen_pos, v_screen_pos)/2.0;
	mediump float vignet_mul = 0.2 + 1.2*(1.0 - 1.3*len + 0.195*len*len); //pow(1.0 - len, 1.3);


#	ifdef FIXED_NOISE
	highp vec2 noise_pos = fract(v_pos.xy/v_pos.w);
#	else
    mediump vec2 v_pos = v_screen_pos + u_offset * 0.05;
    highp vec2 noise_pos = fract(v_pos);
#	endif
	highp vec2 noise_int_pos = floor(noise_pos*1920.0); // TODO(crafn): Get viewport size from uniform
	mediump float noise_mul = mix(0.97, 1.03, rand(noise_int_pos/1000.0)); 
	
	
#	if ENABLE_EDITOR
	color = color_filter(color, u_brightness, u_contrast, u_split_high_tone, u_split_low_tone, u_split_balance, u_saturation, u_hue);
#	endif
	
	color *= vignet_mul;
	color = (vec3(1.0) - (vec3(1.0) - color)*noise_mul);
	//color = (vec3(1.0) - (vec3(1.0) - color)*mix(1.0, noise_mul, step(0.5, sin(u_time*5.0)*0.5 + 0.5)));
#endif
	
#	if 0
    { // Character lighting
        mediump float m = 0.2 + (1.0 - 1.3*len + 0.195*len*len);
        color += 0.01 * pow(m, 20.0);
    }
#	endif
	
    gl_FragColor = vec4(color.rgb, v_color.a/**tex_col.a*/);
	
#else
	gl_FragColor = v_color;
#endif

}
