#define FIXED_NOISE

attribute highp vec3    a_pos;
attribute lowp vec4    a_color;
attribute mediump vec2  a_uv;
attribute lowp float a_flags;

varying mediump vec2 v_uv;
varying mediump vec2 v_screen_pos;
varying mediump vec4 v_color;
#ifdef FIXED_NOISE
    varying highp vec4 v_pos;
#endif


uniform highp mat4      uMVMatrix;
uniform highp mat4      uPMatrix;
uniform mediump float u_depth_offset;
uniform mediump float u_time;

uniform highp vec3      vLight0;

void main(void)
{
	// Bitfields, see VERTEX struct
	float ba = 0.0;
	float b8 = floor(a_flags/256.0 + 0.5); ba += b8*128.0;
#ifndef VERY_LOW_GRAPHICS
	float b7 = floor((a_flags - ba)/128.0 + 0.5); ba += b7*64.0;
	float b6 = floor((a_flags - ba)/64.0 + 0.5); ba += b6*32.0;
	float b5 = floor((a_flags - ba)/32.0 + 0.5); ba += b5*16.0;
	float b4 = floor((a_flags - ba)/16.0 + 0.5); ba += b4*8.0;
	float b3 = floor((a_flags - ba)/8.0 + 0.5); ba += b3*4.0;
	float b2 = floor((a_flags - ba)/4.0 + 0.5); ba += b2*2.0;
	float b1 = floor((a_flags - ba)/2.0 + 0.5);
	float shine_mul = b4;
	float shadowy_mul = b5;
	float lighty_mul = b6;
	float hue_rot_mul = b7;
#endif
	float ortho_mul = b8;
	
    highp vec4 p = vec4(a_pos, 1);
	vec4 pos = uPMatrix * uMVMatrix * p;
	pos.xy *= mix(1.0, -pos.w/uMVMatrix[3][2], ortho_mul);
	v_screen_pos = pos.xy / pos.w;
#ifdef FIXED_NOISE
	v_pos = vec4(a_pos, pos.w);
#endif
	pos.xyz /= pos.w;
	pos.w = 1.0;
	pos.z += u_depth_offset;
    gl_Position = pos;
    v_uv = a_uv;

	lowp vec4 color = a_color;
#ifndef VERY_LOW_GRAPHICS
	// Vertex color animations, translated from render.cpp
	{
		float offset = 1000.0 + u_time*7.0 + a_pos.x*0.5 + a_pos.y*0.2 + a_pos.z*1.0;
		float whole = floor(offset/8.0)*8.0;
		float m = offset - whole;
		float val = max((m - m*m)*4.0, 0.0);
		val = val*val;
		color.r += (color.r*val*0.6 + val*0.1)*shine_mul;
		color.g += (color.g*val*0.6 + val*0.1)*shine_mul;
		color.b += (color.b*val*0.6 + val*0.1)*shine_mul;
	}
	{
		// Same phase with .lighty
		float phase = u_time*1.0;
		float offset = u_time*0.5 + a_pos.x*0.1 + a_pos.y*0.02 + a_pos.z*0.05;
		float val = 0.1*sin(offset)*sin(phase) + 0.9;
		color.r *= mix(1.0, val, lighty_mul);
		color.g *= mix(1.0, val, lighty_mul);
		color.b *= mix(1.0, val, lighty_mul);
	}
	{
		// Same phase with .shadowy
		float phase = u_time*1.0;
		float offset = u_time*0.5 + a_pos.x*0.1 + a_pos.y*0.02 + a_pos.z*0.05;
		float val = 0.1*sin(offset)*sin(phase) + 1.1;
		color.r *= mix(1.0, val, shadowy_mul);
		color.g *= mix(1.0, val, shadowy_mul);
		color.b *= mix(1.0, val, shadowy_mul);
	}
	{
		float phase1 = 1000.0 + u_time*1.0 + pos.x*0.005 + pos.y*0.005 + a_pos.z*0.02;
		float phase2 = phase1 + 3.0;
		float phase3 = phase2 + 3.0;
		
		float whole1 = floor(phase1/10.0)*10.0;
		float whole2 = floor(phase2/10.0)*10.0;
		float whole3 = floor(phase3/10.0)*10.0;
		float m1 = max(1.0 - (phase1 - whole1)/2.0, 0.0);
		float m2 = max(1.0 - (phase2 - whole2)/2.0, 0.0);
		float m3 = max(1.0 - (phase3 - whole3)/2.0, 0.0);
		float v1 = m1*m1*m1*m1;
		float v2 = m2*m2*m2*m2;
		float v3 = m3*m3*m3*m3;

		color.r += v1*0.15*hue_rot_mul;
		color.g += v2*0.15*hue_rot_mul;
		color.b += v3*0.15*hue_rot_mul;
	}
#endif

    v_color = color;
}
