uniform sampler2D image0;
uniform sampler2D image1;
uniform mediump float transition;
uniform int overlay;
uniform lowp float overlay_effect;
uniform highp vec2 scr_resolution;
uniform mediump vec2 fbo_resolution;
uniform mediump float time;
uniform bool reverse_effect;
uniform mediump float trippy;

varying highp vec2 v_uv;

highp float rand(highp vec2 co)
{
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
}

void main()
{
    mediump vec2 img0_uv = v_uv;
    mediump vec2 img1_uv = v_uv;

#if 0 // acid trippy effect
    img0_uv += sin(length(v_uv) * 8.0) * transition * 0.5;
    img1_uv += sin(length(v_uv) * 8.0) * (1.0 - transition) * 0.5;
#endif

    //img1_uv.t += mix(1.0, 0.0, transition);

    mediump vec4 image0_center_rgba = texture2D(image0, img0_uv);
    mediump vec4 image1_center_rgba = texture2D(image1, img1_uv);

    highp vec2 uv = v_uv;
    uv.s += (fbo_resolution.x - scr_resolution.x) / 2.0 / fbo_resolution.x;
    uv.t += (fbo_resolution.y - scr_resolution.y) / 2.0 / fbo_resolution.y;
    uv = 2.0 * uv - vec2(1.0);
    uv.s *= fbo_resolution.x / fbo_resolution.y;  // prevent stretching if fbo.x != fbo.y

#if 0

    // Crossfade
    mediump vec4 center_rgba = mix(image0_center_rgba, image1_center_rgba, transition);

#elif 1

    // Circle
    mediump vec4 center_rgba;
    mediump float rad = length(uv);
    mediump float angle = atan(uv.t / uv.s);
    //angle += 10.0 * (1.0 - transition);
    angle += 2.0 * (1.0 - rad);
    const mediump float fringe_amplitude = 0.15;
    rad += fringe_amplitude * sin(angle * 10.0) + fringe_amplitude;
    rad *= 0.58;
    if (!reverse_effect) {
        mediump float a = rad - transition + 0.1;
        center_rgba = mix(image1_center_rgba, image0_center_rgba, clamp(a * 15.0, 0.0, 1.0));
    } else {
        mediump float a = rad - (1.0 - transition) + 0.1;
        center_rgba = mix(image0_center_rgba, image1_center_rgba, clamp(a * 15.0, 0.0, 1.0));
    }

#elif 1

    // Diagonal slide
    mediump vec4 center_rgba = image0_center_rgba;
    mediump float lim = uv.s - uv.t;
    if (lim + 1.5 < transition * 3.0) {
        center_rgba = image1_center_rgba;
    }

#elif 1

    // Fan
    mediump vec4 center_rgba = image0_center_rgba;
    highp float size = 3.14159265358 / 4.0;
    if (mod(atan(uv.t / uv.s), size) / size < transition) {
        center_rgba = image1_center_rgba;
    }

#endif

    if (overlay == 1) {
        if (overlay_effect == 0.0) {

            center_rgba = mix(image0_center_rgba, image1_center_rgba, image1_center_rgba.a * transition);

        } else if (overlay_effect > 0.0) {

            mediump vec3 blur = vec3(0.0); //image0_center_rgba.rgb;
            mediump vec2 d = vec2(1.0 / scr_resolution.x, 1.0 / scr_resolution.y);
            d *= overlay_effect * 5.0;
            mediump float r = rand(uv) * 3.14159 * 2.0 + time * 1.0;
            //mediump mat2 rot = mat2(vec2(cos(r), sin(r)), vec2(-sin(r), cos(r)));
            //blur += texture2D(image0, v_uv + d*rot*vec2(1.5, 0.1)).rgb;
            if (trippy > 0.0) {
                mediump float freq = 150.0 * sin(trippy * 3.03) + 10.2 * sin(img1_uv.s * 6.2) + 13.2 * cos(img1_uv.s * 5.3);
                mediump vec2 center = scr_resolution / 2.0 / fbo_resolution;
                img1_uv.s += sin(length(v_uv - center) * freq + 3.13 /* sin(time)*/ * (0.7 * sin(time * 0.41))) * overlay_effect * trippy * 0.05;
                img1_uv.t += cos(length(v_uv - center) * freq + 3.13 /* sin(time)*/ * (0.6 * cos(time * 0.27))) * overlay_effect * trippy * 0.05;
            }
            mediump vec2 kern_uv = img1_uv + d*vec2(cos(r), sin(r));
            if (trippy > 0.0) {
                kern_uv.s = mod(kern_uv.s, scr_resolution.x / fbo_resolution.x);
                kern_uv.t = mod(kern_uv.t, scr_resolution.y / fbo_resolution.y);
            }
            blur += texture2D(image0, kern_uv).rgb;
            if (trippy > 0.0) {
                //blur = pow(blur, vec3(mix(1.0, 0.25, trippy)));
                //blur = pow(blur, vec3(mix(1.0, 3.0, trippy)));
                //blur *= vec3(mix(1.0, 0.5, trippy));
                blur += vec3(mix(0.0, 0.5, trippy));
            }
            //blur += texture2D(image0, v_uv + d*rot*vec2(-1.3, -0.2)).rgb;
            //blur += texture2D(image0, v_uv + d*rot*vec2(0.3, -0.8)).rgb;
            //blur += texture2D(image0, v_uv + d*rot*vec2(0.1, 1.1)).rgb;
            //blur += texture2D(image0, v_uv + d*rot*vec2(-1.4, -0.6)).rgb;
            //blur += texture2D(image0, v_uv + d*rot*vec2(1.6, -0.8)).rgb;
            //blur += texture2D(image0, v_uv + d*rot*vec2(1.1, 1.6)).rgb;
            blur /= 1.0;
            image0_center_rgba.rgb = blur;

            //image0_center_rgba.rgb *= mix(1.0, 0.3, transition);  // darken background
            if (trippy == 0.0)
                image0_center_rgba.rgb += mix(vec3(0.0), vec3(0.2), overlay_effect);  // lighten background
            center_rgba = mix(image0_center_rgba, image1_center_rgba, image1_center_rgba.a * transition);

        }
    }

    gl_FragColor = center_rgba;
}
